;********************************************************************
;
;                   JJY Radio Controlled Clock
;              Time signal operation check program
;
;                                             Device : PIC16F877
;                                             Clock  : 12.8MHz
;                                             Author : Seiichi Inoue
;
;********************************************************************

        processor       16F877
        list            f=INHX32
        include         p16f877.inc
        include         picmac1.inc
        radix           dec
        __config        h'3F72' ;OSC is HS
                                ;RB3,RB6,RB7 are I/O
                                ;Brown-out detection ON
                                ;Power-up timer ON
                                ;Code protection OFF
                                ;Data code protection OFF
                                ;Watchdog timer OFF
        errorlevel      -302    ;Eliminate bank warning
        errorlevel      -307    ;Eliminate Setting page bits warning

;************************  Label Definition  ************************
ram_base0       equ     0x20
ram_base1       equ     0xa5    ;0-4 are used as common save area
ram_base2       equ     0x125   ;0-4 are used as common save area
rxbflen         equ     28      ;Receive buffer length

#define	det_mark_1st    dec_sts1,1      ;1st 0 sec maker detect flag
#define	dec_time_1st    dec_sts1,4      ;1st time decode flag

;@@@ 7 segment LED data
seg7_0  equ     b'00111111'     ;-gfedcba Pattern 0
seg7_1  equ     b'00000110'     ;         Pattern 1
seg7_2  equ     b'01011011'     ;         Pattern 2
seg7_3  equ     b'01001111'     ;         Pattern 3
seg7_4  equ     b'01100110'     ;         Pattern 4
seg7_5  equ     b'01101101'     ;         Pattern 5
seg7_6  equ     b'01111101'     ;         Pattern 6
seg7_7  equ     b'00000111'     ;         Pattern 7
seg7_8  equ     b'01111111'     ;         Pattern 8
seg7_9  equ     b'01101111'     ;         Pattern 9
seg7_a  equ     b'01000000'     ;         Hyphen
seg7_b  equ     b'00000000'     ;         OFF

;*** Register area definition (bank 0)
        cblock	ram_base0
        tmp_w                   ;Working register save area
        tmp_sts                 ;Status register save area
        tmp_pclath              ;Pclath register save area
        tmp_fsr                 ;FSR register save area
        tmp_fsr2                ;FSR register save area 2
        freg0                   ;Parameter register 0
        freg1                   ;Parameter register 1
        freg2                   ;Parameter register 2
        tco_code                ;TCO code 0:0, 1:1, 2:mark
        tco_valid               ;TCO valid flag

        mark_cnt                ;marker counter
        mark_pd_cnt             ;marker period counter
        zero_cnt                ;0 code counter
        tm_chk_cnt_wd           ;Time check counter (Week)
        tm_chk_cnt_y            ;Time check counter (Year)
        tm_chk_cnt_nd           ;Time check counter (Day)
        tm_chk_cnt_h            ;Time check counter (Hour)
        tm_chk_cnt_m            ;Time check counter (Munite)
        dctm_wd                 ;Week decode counter
        dctm_100nd              ;100th of days decode counter

        dctm_10nd               ;10th of days decode counter
        dctm_1nd                ;1st of day  decode counter
        dctm_10y                ;10th of years decode counter
        dctm_1y                 ;1st of year decode counter
        dctm_10h                ;10th of hours decode counter
        dctm_1h                 ;1st of hour decode counter
        dctm_10m                ;10th of minutes decode conuter
        dctm_1m                 ;1st of minute decode counter
        bk_dctm_uy              ;Leap year backup counter
        bk_dctm_wd              ;Week backup counter

        bk_dctm_100nd           ;100th of days backup counter
        bk_dctm_10nd            ;10th of days backup counter
        bk_dctm_1nd             ;1st of day backup counter
        bk_dctm_10y             ;10th of years backup counter
        bk_dctm_1y              ;1st of year backup counter
        bk_dctm_10h             ;10th of hours backup counter
        bk_dctm_1h              ;1st of hour backup counter
        bk_dctm_10m             ;10th of minutes backup conuter
        bk_dctm_1m              ;1st of minute backup counter
        dec_min_cnt             ;Decoded time (Every minute)

        xpos                    ;X position on LCD
        ypos                    ;Y position on LCD
        acch                    ;Accumulate register A (High)
        accm                    ;Accumulate register A (Mid)
        accl                    ;Accumulate register A (Low)
        bcch                    ;Accumulate register B (High)
        bccm                    ;Accumulate register B (Mid)
        bccl                    ;Accumulate register B (Low)
        acc_flg                 ;Accumulating flag
        porta_buf               ;Port A buffer

        portc_buf               ;Port C buffer
        tled_cnt                ;LED turn on couner
        err_cnth                ;Error counter (Upper)
        err_cntl                ;Error counter (Lower)
        key_sts                 ;Key status
        key_dat                 ;Key number
        cur_timer               ;Current timer
        tm_mode                 ;Mode(0:off/1:continue/2:single)
        tm_hour                 ;Hour timer
        tm_minute               ;Munite timer

        tm_second               ;Second timer
        tm_index                ;Timer setting item index
        reserve                 ;Reserved for debugging
        k_tmout_cnt             ;Time-out counter for key proc
        tm_limit                ;Upper limit couner
        disp_flag               ;Display control flag
        disp_mode               ;Display mode
        rx_sts                  ;Receiver status
        rx_cc                   ;Received data counter
        rx_ptr                  ;Received data pointer

        unadj_cnt               ;0 second unadjust counter
        tmbf_mode               ;Timer set/check (Mode)
        tmbf_str_hour           ;Timer start set/check (Hour)
        tmbf_str_minute         ;Timer start set/check (Munite)
        tmbf_str_second         ;Timer start set/check (Second)
        tmbf_stp_hour           ;Timer stop set/check (Hour)
        tmbf_stp_minute         ;Timer stop set/check (Munite)
        tmbf_stp_second         ;Timer stop set/check (Second)
        dec_sts2                ;decode status2
        tm_uy                   ;Leap year flag
;
;*** Common data for all banks
        dec_sts1                ;decode status1
        tm_ndh                  ;Day counter (Upper)
        tm_ndl                  ;Day counter (Lower)
        tm_10y                  ;10th of years counter
        tm_1y                   ;1st of year counter
        tm_10mth                ;10th of mounths counter
        tm_1mth                 ;1st of mounth counter
        tm_10d                  ;10th of days counter
        tm_1d                   ;1st of day counter
        tm_10h                  ;10th of hours counter
        tm_1h                   ;1st of hour counter
        tm_10m                  ;10th of minutes conuter
        tm_1m                   ;1st of minute counter
        tm_10s                  ;10th of seconds counter
        tm_1s                   ;1st of second counter
        tm_wd                   ;Week counter
        endc
;
;*** Register area definition (bank 1) 80 bytes can be used
        cblock	ram_base1
        freg10                  ;Parameter register 10
        freg11                  ;Parameter register 11
        freg12                  ;Parameter register 12
        int_cnt                 ;Timer 2 interval for 100uS
        int_cnt10m              ;Timer 2 interval for 10mS
        int_cnt100m             ;Timer 2 interval for 100mS
        delay_cnt100u           ;100uS delay counter
        delay_cnt10m            ;10mS delay counter
        tco_cnt                 ;TCO counter
        tco_hicnt               ;TCO high counter

        tco_period              ;TCO period counter
        tco_sts                 ;TCO status
        sync_cnt                ;0 code counter for Sync detect
        ikey_dat                ;Key number
        ikey_dat0               ;Key scan position
        ikey_dat1               ;Key input data
        key_phs                 ;Key phase
        rep_cnt                 ;Key repeate counter
        eep_add                 ;EEPROM address
        eep_data                ;EEPROM data

        rx_dat:rxbflen          ;Received data buffer(28 bytes)

;@@@ Register area for 7 seg LED
        led_ha                  ;Table head address
        led_0                   ;7seg pattern 0
        led_1                   ;7seg pattern 1
        led_2                   ;7seg pattern 2
        led_3                   ;7seg pattern 3
        led_4                   ;7seg pattern 4
        led_5                   ;7seg pattern 5
        led_6                   ;7seg pattern 6
        led_7                   ;7seg pattern 7
        led_8                   ;7seg pattern 8

        led_9                   ;7seg pattern 9
        led_a                   ;7seg hyphen
        led_b                   ;7seg OFF
        led_10y                 ;10th of year
        led_1y                  ;1st of year
        led_10mth               ;10th of month
        led_1mth                ;1st of month
        led_10d                 ;10th of day
        led_1d                  ;1st of day
        led_10h                 ;10th of hour

        led_1h                  ;1st of hour
        led_10m                 ;10th of minute
        led_1m                  ;1st of minute
        led_10s                 ;10th of second
        led_1s                  ;1st of second
        led_wd                  ;Week of the day
        endc

;*** Register area definition (bank 2) 80 bytes can be used
        cblock	ram_base2
        led_hyphen              ;0:Hyphen OFF, 1:Hyphen ON
        led_cnt100u             ;100us counter
        led_wkdata              ;LED work data
        led_ff                  ;Fast forward data
        led_fsr                 ;FSR save area
        stm                     ;1Sec software timer
        stm100                  ;100mS software timer
        endc

seg7_ha equ     led_0           ;LED pattern table head address

;********************************************************************
;                           Program Start
;********************************************************************
        org     0               ;Reset Vector
        lgoto   init
        org     4               ;Interrupt Vector
        lgoto   int


;####################################################################
;                          Initial routine
;####################################################################
        org     0x800
init
        setpch
;*** Port initialization
        rbank1
        clrf    trisa           ;Set RA5-0 to OUTPUT
        bsf     trisa,4         ;Set RA4 to INPUT
        movlw   b'11111111'     ;Set input data
        movwf   trisb           ;Set RB7-0 to INPUT
        clrf    trisc           ;Set RC7-0 to OUTPUT
        clrf    trisd           ;Set RD7-0 to OUTPUT
        clrf    trise           ;Set RC2-0 to OUTPUT

;*** Set Option reg
        movlw   b'00000111'     ;PS=1:256
        movwf   option_reg      ;Set OPTION_REG

;*** ADCON1 set
        movlw   b'00000110'     ;Set all digital port
        movwf   adcon1          ;Set ADCON1
        rbank0

;*** PORT set
        clrf    porta           ;Clear PORTA
        clrf    portc           ;Clear PORTC
        clrf    portd           ;Clear PORTD
        clrf    porte           ;Clear PORTE

;*** Set TMR0 reg
        movlw   d'6'            ;256-20000/80us=6
        movwf   tmr0            ;Set TMR0

;*** Display data initialization
        movlw   d'0'            ;(2004/12/31 23:59:00 FRI)
        movwf   tm_10y
        movlw   d'4'
        movwf   tm_1y
        movlw   d'1'
        movwf   tm_10mth
        movlw   d'2'
        movwf   tm_1mth
        movlw   d'3'
        movwf   tm_10d
        movlw   d'1'
        movwf   tm_1d
        movlw   d'2'
        movwf   tm_10h
        movlw   d'3'
        movwf   tm_1h
        movlw   d'5'
        movwf   tm_10m
        movlw   d'9'
        movwf   tm_1m
        clrf    tm_10s
        clrf    tm_1s
        movlw   d'5'
        movwf   tm_wd

        rbank1
        clrf    led_10y         ;Clear LED 10th of year
        clrf    led_1y          ;Clear LED 1st of year
        clrf    led_10mth       ;Clear LED 10th of month
        clrf    led_1mth        ;Clear LED 1st of month
        clrf    led_10d         ;Clear LED 10th of day
        clrf    led_1d          ;Clear LED 1st of day
        clrf    led_10h         ;Clear LED 10th of hour
        clrf    led_1h          ;Clear LED 1st of hour
        clrf    led_10m         ;Clear LED 10th of minute
        clrf    led_1m          ;Clear LED 1st of minute
        clrf    led_10s         ;Clear LED 10th of second
        clrf    led_1s          ;Clear LED 1st of second
        clrf    led_wd          ;Clear LED week of the day
        movlw   seg7_ha         ;Set pattern table head address
        movwf   led_ha          ;Save pattern table head address
        movlw   seg7_0          ;Set pattern 0
        movwf   led_0           ;Save pattern 0
        movlw   seg7_1          ;Set pattern 1
        movwf   led_1           ;Save pattern 1
        movlw   seg7_2          ;Set pattern 2
        movwf   led_2           ;Save pattern 2
        movlw   seg7_3          ;Set pattern 3
        movwf   led_3           ;Save pattern 3
        movlw   seg7_4          ;Set pattern 4
        movwf   led_4           ;Save pattern 4
        movlw   seg7_5          ;Set pattern 5
        movwf   led_5           ;Save pattern 5
        movlw   seg7_6          ;Set pattern 6
        movwf   led_6           ;Save pattern 6
        movlw   seg7_7          ;Set pattern 7
        movwf   led_7           ;Save pattern 7
        movlw   seg7_8          ;Set pattern 8
        movwf   led_8           ;Save pattern 8
        movlw   seg7_9          ;Set pattern 9
        movwf   led_9           ;Save pattern 9
        movlw   seg7_a          ;Set hyphen pattern
        movwf   led_a           ;Save hyphen pattern
        movlw   seg7_b          ;Set OFF pattern
        movwf   led_b           ;Save OFF pattern

        bsf     det_mark_1st    ;Set 1st 0 sec maker detect flag
        bsf     dec_time_1st    ;Set 1st time decode flag

        rbank2
        clrf    led_hyphen      ;Clear LED hyphen
        clrf    led_cnt100u     ;Clear LED 100us counter
        clrf    led_wkdata      ;Clear LED work data

;*** Timer counter
        movlw   d'50'           ;Set 1 second
        movwf   stm             ;Set software timer

;*** Set interrupt condition
        rbank0                  ;Set bank 0
        movlw   b'10100000'     ;GIE,TOIE,INTE=1
        movwf   intcon          ;Interruption enable

        goto    $               ;Wait interruption


;####################################################################
;                      Interruption processing
;####################################################################
int
        setpch
        rbank0
;********** TMR0 Interruption ( 20 milliseconds interval) ***********
        bcf     intcon,t0if     ;Clear timer int flag
        movlw   d'6'            ;Set 20mS
        movwf   tmr0            ;Set TMR0

        btfsc   portc,3         ;Time sgnal ON ?
        goto    int02           ;Yes.
        rbank2
        movlw   d'5'            ;Set 100 milliseconds
        movwf   stm100          ;Set timer
int01
        decfsz  stm,f           ;Software timer - 1
        goto    int_end         ;Jump to interruption end
        movlw   d'50'           ;Set 1 second
        movwf   stm             ;Set software timer
        goto    inc00           ;Jump to 1 second process
int02
        rbank2
        decfsz  stm100,f        ;100mS timer - 1
        goto    int01           ;Jump to 1 second check
        rbank0
        bcf     portc,3         ;Time signal OFF
        rbank2
        goto    int01           ;Jump to 1 second check

;********************** 1 second interval ***************************
inc00
        incf    tm_1s,f         ;1st of second + 1
        movlw   d'10'           ;Set check data
        subwf   tm_1s,w         ;Overflow check
        btfss   status,z        ;1st of second overflow ?
        goto    display         ;No.
        clrf    tm_1s           ;Clear 1st of second
        incf    tm_10s,f        ;10th of second + 1
        movlw   d'6'            ;Set check data
        subwf   tm_10s,w        ;Overflow check
        btfss   status,z        ;10th of second overflow ?
        goto    display         ;No.
        clrf    tm_10s          ;Clear 10th of second
        incf    tm_1m,f         ;1st of minute + 1
        movlw   d'10'           ;Set check data
        subwf   tm_1m,w         ;Overflow check
        btfss   status,z        ;1st of minute overflow ?
        goto    display         ;No.
        clrf    tm_1m           ;Clear 1st of minute
        incf    tm_10m,f        ;10th of minute + 1
        movlw   d'6'            ;Set check data
        subwf   tm_10m,w        ;Overflow check
        btfss   status,z        ;10th of minute overflow ?
        goto    display         ;No.
        clrf    tm_10m          ;Clear 10th of minute
        incf    tm_1h,f         ;1st of hour + 1
        movfw   tm_10h          ;Read 10th of hour
        btfss   status,z        ;0X hour ?
        goto    inc02           ;No.
inc01
        movlw   d'10'           ;Set check data
        subwf   tm_1h,w         ;Overflow check
        btfss   status,z        ;1st of hour overflow ?
        goto    display         ;No.
        clrf    tm_1h           ;Clear 1st of hour
        incf    tm_10h,f        ;10th of hour + 1
        goto    display
inc02
        movlw   d'1'            ;Set check data
        subwf   tm_10h,w        ;Overflow check
        btfsc   status,z        ;1X hour ?
        goto    inc01           ;Yes.
        movlw   d'4'            ;Set check data
        subwf   tm_1h,w         ;Overflow check
        btfss   status,z        ;24 o'clock ?
        goto    display         ;No.
        clrf    tm_1h           ;Clear 1st of hour
        clrf    tm_10h          ;Clear 10th of hour
        movlw   d'1'            ;Set 2005/01/01
        movwf   tm_1d
        movwf   tm_1mth
        clrf    tm_10d
        clrf    tm_10mth
        incf    tm_1y,f
        incf    tm_wd,f         ;Week of the day + 1
        movlw   d'7'            ;Set check data
        subwf   tm_wd,w         ;Overflow check
        btfss   status,z        ;Week overflow ?
        goto    display         ;No.
        clrf    tm_wd           ;Set Monday

display
        lcall   led_disp        ;7 seg LED Display subroutine
        setpch

;********************************************************************
;                    End of interruption process
;********************************************************************
int_end
        retfie

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
;                   7 seg LED Display subroutine
;                                             Author : Seiichi Inoue
;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        org     0x1000
led_disp
        rbank1
        btfsc   det_mark_1st    ;1st marker detected ?
        goto    led_dsp03       ;Yes. Jump to time decode check
led_dsp01
        rbank2
        btfsc   led_hyphen,0    ;LED hyphen ON ?
        goto    led_dsp02       ;Yes.
        rbank1
        movlw   d'10'           ;Read LED hyphen data
        movwf   led_10y         ;Set 10th of year
        movwf   led_1y          ;Set 1st of year
        movwf   led_10mth       ;Set 10th of month
        movwf   led_1mth        ;Set 1st of month
        movwf   led_10d         ;Set 10th of day
        movwf   led_1d          ;Set 1st of day
        movwf   led_10h         ;Set 10th of hour
        movwf   led_1h          ;Set 1st of hour
        movwf   led_10m         ;Set 10th of minute
        movwf   led_1m          ;Set 1st of minute
        movwf   led_10s         ;Set 10th of second
        movwf   led_1s          ;Set 1st of second
        clrf    led_wd          ;Set week OFF
        rbank2
        incf    led_hyphen,f    ;Set hyphen ON
        goto    led_cont        ;Jump to LED control

led_dsp02
        rbank1
        movlw   d'11'           ;Read LED OFF data
        movwf   led_10y         ;Set 10th of year
        movwf   led_1y          ;Set 1st of year
        movwf   led_10mth       ;Set 10th of month
        movwf   led_1mth        ;Set 1st of month
        movwf   led_10d         ;Set 10th of day
        movwf   led_1d          ;Set 1st of day
        movwf   led_10h         ;Set 10th of hour
        movwf   led_1h          ;Set 1st of hour
        movwf   led_10m         ;Set 10th of minute
        movwf   led_1m          ;Set 1st of minute
        movwf   led_10s         ;Set 10th of second
        movwf   led_1s          ;Set 1st of second
        clrf    led_wd          ;Set week OFF
        rbank2
        clrf    led_hyphen      ;Set hyphen OFF
        goto    led_cont        ;Jump to LED control

led_dsp03
        btfss   dec_time_1st    ;1st time decoded ?
        goto    led_dsp01       ;No. Jump to LED hyphen

led_time_read:
        movfw   tm_10y          ;Read 10th of year
        movwf   led_10y         ;Save 10th of year
        movfw   tm_1y           ;Read 1st of year
        movwf   led_1y          ;Save 1st of year
        movfw   tm_10mth        ;Read 10th of month
        btfsc   status,z        ;10th of month = 0 ?
        movlw   d'11'           ;Yes. Set LED OFF data
        movwf   led_10mth       ;Save 10th of month
        movfw   tm_1mth         ;Read 1st of month
        movwf   led_1mth        ;Save 1st of month
        movfw   tm_10d          ;Read 10th of day
        btfsc   status,z        ;10th of day = 0 ?
        movlw   d'11'           ;Yes. Set LED OFF data
        movwf   led_10d         ;Save 10th of day
        movfw   tm_1d           ;Read 1st of day
        movwf   led_1d          ;Save 1st of day
        movfw   tm_10h          ;Read 10th of hour
        btfsc   status,z        ;10th of hour = 0 ?
        movlw   d'11'           ;Yes. Set LED OFF data
        movwf   led_10h         ;Save 10th of hour
        movfw   tm_1h           ;Read 1st of hour
        movwf   led_1h          ;Save 1st of hour
;*** 24H -> 12H converting
        movlw   d'11'           ;Set check data
        subwf   led_10h,w       ;10th of hour - OFF data
        btfsc   status,z        ;Hour = 0X ?
        goto    led_time05      ;Yes.
        decfsz  led_10h,w       ;10th of hour = 1 ?
        goto    led_time01      ;No.
        movfw   led_1h          ;Read 1st of hour
        btfsc   status,z        ;Hour = 10 o'clock ?
        goto    led_time05      ;Yes.
        movlw   d'1'            ;Set check data
        subwf   led_1h,w        ;1st of hour - 1
        btfsc   status,z        ;Hour = 11 o'clock ?
        goto    led_time05      ;Yes.
        movlw   d'2'            ;Set check data
        subwf   led_1h,w        ;1st of hour - 2
        btfsc   status,z        ;Hour = 12 o'clock ?
        goto    led_time05      ;Yes.
        movlw   d'11'           ;Set LED OFF data
        movwf   led_10h         ;10th of hour LED OFF
        movlw   d'2'            ;Set compensation data
        subwf   led_1h,f        ;Set 1st of hour
        goto    led_time05
led_time01
        movlw   d'2'            ;Set check data
        subwf   led_10h,w       ;10th of hour - 2
        btfss   status,z        ;Hour = 2X ?
        goto    led_time05      ;No.
        movfw   led_1h          ;Read 1st of hour
        btfss   status,z        ;Hour = 20 o'clock ?
        goto    led_time02      ;No.
        movlw   d'11'           ;Set LED OFF data
        movwf   led_10h         ;10th of hour LED OFF
        movlw   d'8'            ;Set 8 data
        movwf   led_1h          ;Set 8 o'clock
        goto    led_time05
led_time02
        decfsz  led_1h,w        ;Hour = 21 o'clock ?
        goto    led_time03      ;No.
        movlw   d'11'           ;Set LED OFF data
        movwf   led_10h         ;10th of hour LED OFF
        movlw   d'9'            ;Set 9 data
        movwf   led_1h          ;Set 9 o'clock
        goto    led_time05
led_time03
        movlw   d'2'            ;Set check data
        subwf   led_1h,w        ;1st of hour - 2
        btfss   status,z        ;Hour = 22 o'clock ?
        goto    led_time04      ;No.
        movlw   d'1'            ;Set 1 data
        movwf   led_10h         ;Set 10th of hour = 1
        clrf    led_1h          ;Hour = 10 o'clock
        goto    led_time05
led_time04
        movlw   d'1'            ;Set 1 data
        movwf   led_10h         ;Set 10th of hour = 1
        movwf   led_1h          ;Hour = 11 o'clock
led_time05
        movfw   tm_10m          ;Read 10th of minute
        movwf   led_10m         ;Save 10th of minute
        movfw   tm_1m           ;Read 1st of minute
        movwf   led_1m          ;Save 1st of minute
        movfw   tm_10s          ;Read 10th of second
        movwf   led_10s         ;Save 10th of second
        movfw   tm_1s           ;Read 1st of second
        movwf   led_1s          ;Save 1st of second
        incf    tm_wd,w         ;Week + 1
        movwf   led_wd          ;Save week

;*** Time signal control
        movlw   d'6'            ;Set 6 o'clock
        subwf   led_1m,w        ;Check hour                 @@@@
        btfss   status,z        ;6 o'clock ?
        goto    led_sig01       ;No.
        call    true_check      ;Check true time
        goto    led_cont        ;Jump to LED control
led_sig01
        movlw   d'7'            ;Set 7 o'clock
        subwf   led_1m,w        ;Check hour                 @@@@
        btfss   status,z        ;7 o'clock ?
        goto    led_sig02       ;No.
        call    true_check      ;Check true time
        goto    led_cont        ;Jump to LED control
led_sig02
        movlw   d'8'            ;Set 8 o'clock
        subwf   led_1m,w        ;Check hour                 @@@@
        btfss   status,z        ;8 o'clock ?
        goto    led_sig03       ;No.
        call    true_check      ;Check true time
        goto    led_cont        ;Jump to LED control
led_sig03
        movlw   d'9'            ;Set 9 o'clock
        subwf   led_1m,w        ;Check hour                 @@@@
        btfss   status,z        ;9 o'clock ?
        goto    led_sig04       ;No.
        call    true_check      ;Check true time
        goto    led_cont        ;Jump to LED control

led_sig04
        movlw   d'11'           ;Set check data
        subwf   led_10mth,w     ;Check month
        btfss   status,z        ;Month = 0X ?
        goto    led_cont        ;No.
        decfsz  led_1mth,w      ;Month = January ?
        goto    led_cont        ;No.
        movlw   d'11'           ;Set check data
        subwf   led_10d,w       ;Check day
        btfss   status,z        ;Day = 0X ?
        goto    led_cont        ;No.
        decfsz  led_1d,w        ;January 1st ?
        goto    led_cont        ;No.
        movlw   d'11'           ;Set check data
        subwf   led_10h,w       ;Check hour
        btfss   status,z        ;Hour = 0X ?
        goto    led_cont        ;No.
        movfw   led_1h          ;Read 1st of hour
        btfss   status,z        ;0 o'clock ?
        goto    led_cont        ;No.
        movfw   led_10m         ;Read 10th of minute
        btfss   status,z        ;Minute = 0X ?
        goto    led_cont        ;No.
        movfw   led_1m          ;Read 1st of minute
        btfss   status,z        ;Minute = 00 ?
        goto    led_sig05       ;No.
        movfw   led_10s         ;Read 10th of second
        btfss   status,z        ;Second = 0X ?
        goto    led_cont        ;No.
        movfw   led_1s          ;Read 1st of second
        btfss   status,z        ;Second = 00 ?
        goto    led_cont        ;No.
        rbank0
        bsf     portc,2         ;Set chime
        bsf     portc,3         ;Start signal
        rbank2
        movlw   d'7'            ;Set fast forward data
        movwf   led_ff          ;Save fast forward
        goto    led_cont        ;Jump to LED control
led_sig05
        decf    led_1m,w        ;1st of minute - 1
        btfss   status,z        ;1/1 00:01 ?
        goto    led_cont        ;No.
        rbank2
        movfw   led_ff          ;Read fast forward
        btfsc   status,z        ;Fast forward ended ?
        goto    led_sig06       ;Yes.
        rbank0
        bcf     portc,2         ;Set melody
        bsf     portc,4         ;Set silent
        bsf     portc,3         ;Fast forward
        rbank2
        decf    led_ff,f        ;Fast forward counter - 1
        goto    led_cont        ;Jump to LED control
led_sig06
        rbank0
        bcf     portc,4         ;Cancel silent

;*** Display control
led_cont
        rbank0
        movlw   b'00000011'     ;Set 10th of year selector
        movwf   porte           ;Set selector data
        rbank1
        movlw   b'10000000'     ;Set 10th of year selector
        iorwf   led_10y,w       ;Set data
        call    led_write       ;Write data

        movlw   b'00000010'     ;Set 1st of year selector
        movwf   porte           ;Set selector data
        rbank1
        movlw   b'10000000'     ;Set 1st of year selector
        iorwf   led_1y,w        ;Set data
        call    led_write       ;Write data

        movlw   b'00000001'     ;Set 10th of month selector
        movwf   porte           ;Set selector data
        rbank1
        movlw   b'10000000'     ;Set 10th of month selector
        iorwf   led_10mth,w     ;Set data
        call    led_write       ;Write data

        movlw   b'00000000'     ;Set 1st of month selector
        movwf   porte           ;Set selector data
        rbank1
        movlw   b'10000000'     ;Set 1st of month selector
        iorwf   led_1mth,w      ;Set data
        call    led_write       ;Write data

        movlw   b'00000111'     ;Set 10th of day selector
        movwf   porte           ;Set selector data
        rbank1
        movfw   led_10d         ;Read 10th of day
        call    led_write       ;Write data

        movlw   b'00000110'     ;Set 1st of day selector
        movwf   porte           ;Set selector data
        rbank1
        movfw   led_1d          ;Read 1st of day
        call    led_write       ;Write data

        movlw   b'00000011'     ;Set 10th of hour selector
        movwf   porte           ;Set selector data
        rbank1
        movfw   led_10h         ;Read 10th of hour
        call    led_write       ;Write data

        movlw   b'00000010'     ;Set 1st of hour selector
        movwf   porte           ;Set selector data
        rbank1
        movfw   led_1h          ;Read 1st of hour
        call    led_write       ;Write data

        movlw   b'00000001'     ;Set 10th of minute selector
        movwf   porte           ;Set selector data
        rbank1
        movfw   led_10m         ;Read 10th of minute
        call    led_write       ;Write data

        movlw   b'00000000'     ;Set 1st of minute selector
        movwf   porte           ;Set selector data
        rbank1
        movfw   led_1m          ;Read 1st of minute
        call    led_write       ;Write data

        movlw   b'00000100'     ;Set week selector
        movwf   porte           ;Set selector data
        rbank1
        movlw   b'10000000'     ;Set week selector
        iorwf   led_wd,w        ;Set data
        rbank0
        movwf   portd           ;Write data
        bsf     portc,5         ;Start clock pulse
        rbank1
        call    led_t100u       ;100usec wait
        rbank0
        bcf     portc,5         ;Stop clock pulse (Data latch)
        rbank1
        call    led_t100u       ;100usec wait
        rbank0

        movlw   b'00000101'     ;Set 10th of second selector
        movwf   porte           ;Set selector data
        rbank1
        movfw   led_10s         ;Read 10th of second
        call    led_write       ;Write data

        movlw   b'00000100'     ;Set 1st of second selector
        movwf   porte           ;Set selector data
        rbank1
        movfw   led_1s          ;Read 1st of second
        call    led_write       ;Data write

        rbank0
        return

;@@@@@@@@@@ True time check and Signal start Subroutine @@@@@@@@@@@@@
true_check
;        movfw   led_10m         ;Read 10th of minute       @@@@
;        btfss   status,z        ;Minute = 0X ?             @@@@
;        goto    true_end        ;No.                       @@@@
;        movfw   led_1m          ;Read 1st of minute        @@@@
;        btfss   status,z        ;Minute = 00 ?             @@@@
;        goto    true_end        ;No.                       @@@@
        movfw   led_10s         ;Read 10th of second
        btfss   status,z        ;Second = 0X ?
        goto    true_end        ;No.
        movfw   led_1s          ;Read 1st of second
        btfss   status,z        ;Second = 00 ?
        goto    true_end        ;No.
        rbank0
        bsf     portc,3         ;Start signal
true_end
        return

;@@@@@@@@@@@@@@@@@@@@ Data write Subroutine @@@@@@@@@@@@@@@@@@@@@@@@@
led_write
        rbank2
        movwf   led_wkdata      ;Save input data
        movfw   fsr             ;Read FSR
        movwf   led_fsr         ;Save FSR
        movlw   b'00001111'     ;Set mask
        andwf   led_wkdata,w    ;Pickup LED data
        rbank1
        addwf   led_ha,w        ;LED pattern table HA + LED data
        movwf   fsr             ;Set table address
        movfw   indf            ;Read 7seg data
        rbank0
        movwf   portd           ;Set 7seg data
        rbank2
        movfw   led_fsr         ;Read saved FSR
        movwf   fsr             ;Recover FSR
        movlw   b'10000000'     ;Set mask
        andwf   led_wkdata,w    ;Pick up selector data
        rbank0
        iorwf   portd,f         ;Set selector data
        bsf     portc,5         ;Start clock pulse
        rbank2
        call    led_t100u       ;100usec wait
        rbank0
        bcf     portc,5         ;Stop clock pulse (Data latch)
        rbank2
        call    led_t100u       ;100usec wait
        rbank0
        return

;@@@@@@@@@@@@@@@@@@@ 100usec Timer Subroutine @@@@@@@@@@@@@@@@@@@@@@@
led_t100u
        movlw   d'63'           ;Set loop cnt
        movwf   led_cnt100u     ;Save loop cnt
led_t100ulp
        nop                     ;Time adjust
        nop                     ;Time adjust
        decfsz  led_cnt100u,f   ;cnt100u - 1 = 0 ?
        goto    led_t100ulp     ;No, continue
        return 

;********************************************************************
;                 END of Clock display check program
;********************************************************************

       end
